# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .action_form_attachment import ActionFormAttachment
from .action_form_request_param_value import ActionFormRequestParamValue
from .ask_object_request import AskObjectRequest
from .ask_request import AskRequest
from .ask_stream_action_event import AskStreamActionEvent
from .ask_stream_chart_event import AskStreamChartEvent
from .ask_stream_end_event import AskStreamEndEvent
from .ask_stream_metadata_event import AskStreamMetadataEvent
from .ask_stream_o_auth_button_event import AskStreamOAuthButtonEvent
from .ask_stream_start_event import AskStreamStartEvent
from .ask_stream_text_event import AskStreamTextEvent
from .categorization_response import CategorizationResponse
from .conversation_field import ConversationField
from .conversation_filter import ConversationFilter
from .conversation_message_request import ConversationMessageRequest
from .conversation_metadata import ConversationMetadata
from .conversation_patch_request import ConversationPatchRequest
from .conversation_request import ConversationRequest
from .conversations_response import ConversationsResponse
from .conversations_search_request import ConversationsSearchRequest
from .deliver_conversation_message_request import DeliverConversationMessageRequest
from .deliver_message_request import (
    DeliverMessageRequest,
    DeliverMessageRequest_Conversation,
    DeliverMessageRequest_User,
)
from .deliver_message_response import DeliverMessageResponse
from .deliver_user_message_request import DeliverUserMessageRequest
from .delivery_status import DeliveryStatus
from .feedback_field import FeedbackField
from .feedback_filter import FeedbackFilter
from .feedback_request import FeedbackRequest
from .numeric_conversation_field import NumericConversationField
from .object_stream_response import (
    ObjectStreamResponse,
    ObjectStreamResponse_End,
    ObjectStreamResponse_Start,
    ObjectStreamResponse_Text,
)
from .resolution_status import ResolutionStatus
from .simulation_filter import SimulationFilter
from .stream_response import (
    StreamResponse,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_End,
    StreamResponse_Metadata,
    StreamResponse_OauthButton,
    StreamResponse_Start,
    StreamResponse_Text,
)
from .submit_action_form_request import SubmitActionFormRequest
from .update_metadata_request import UpdateMetadataRequest

__all__ = [
    "ActionFormAttachment",
    "ActionFormRequestParamValue",
    "AskObjectRequest",
    "AskRequest",
    "AskStreamActionEvent",
    "AskStreamChartEvent",
    "AskStreamEndEvent",
    "AskStreamMetadataEvent",
    "AskStreamOAuthButtonEvent",
    "AskStreamStartEvent",
    "AskStreamTextEvent",
    "CategorizationResponse",
    "ConversationField",
    "ConversationFilter",
    "ConversationMessageRequest",
    "ConversationMetadata",
    "ConversationPatchRequest",
    "ConversationRequest",
    "ConversationsResponse",
    "ConversationsSearchRequest",
    "DeliverConversationMessageRequest",
    "DeliverMessageRequest",
    "DeliverMessageRequest_Conversation",
    "DeliverMessageRequest_User",
    "DeliverMessageResponse",
    "DeliverUserMessageRequest",
    "DeliveryStatus",
    "FeedbackField",
    "FeedbackFilter",
    "FeedbackRequest",
    "NumericConversationField",
    "ObjectStreamResponse",
    "ObjectStreamResponse_End",
    "ObjectStreamResponse_Start",
    "ObjectStreamResponse_Text",
    "ResolutionStatus",
    "SimulationFilter",
    "StreamResponse",
    "StreamResponse_Action",
    "StreamResponse_Chart",
    "StreamResponse_End",
    "StreamResponse_Metadata",
    "StreamResponse_OauthButton",
    "StreamResponse_Start",
    "StreamResponse_Text",
    "SubmitActionFormRequest",
    "UpdateMetadataRequest",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .action_base import ActionBase
from .action_enum_option import ActionEnumOption
from .action_execution_param_value import (
    ActionExecutionParamValue,
    ActionExecutionParamValue_Attachment,
    ActionExecutionParamValue_Primitive,
)
from .action_form_field import ActionFormField
from .action_o_auth_configuration import ActionOAuthConfiguration
from .action_parameter import ActionParameter
from .action_parameter_type import ActionParameterType
from .action_response import ActionResponse
from .app_precondition import AppPrecondition
from .app_user import AppUser
from .app_user_identifier import AppUserIdentifier
from .app_user_identifying_property_type import AppUserIdentifyingPropertyType
from .app_user_request import AppUserRequest
from .app_user_response import AppUserResponse
from .attachment_request import AttachmentRequest
from .attachment_response import AttachmentResponse
from .attachment_status import AttachmentStatus
from .base_attachment import BaseAttachment
from .base_conversation_response import BaseConversationResponse
from .base_paginated_request import BasePaginatedRequest
from .base_search_request import BaseSearchRequest
from .bot_action_form_response import BotActionFormResponse
from .bot_chart_response import BotChartResponse
from .bot_conversation_message_type import BotConversationMessageType
from .bot_logic import BotLogic
from .bot_logic_action_executed_detail import BotLogicActionExecutedDetail
from .bot_logic_action_item import BotLogicActionItem
from .bot_logic_action_reviewed_detail import BotLogicActionReviewedDetail
from .bot_logic_form_submission_item import BotLogicFormSubmissionItem
from .bot_logic_item import (
    BotLogicItem,
    BotLogicItem_Actions,
    BotLogicItem_Form,
    BotLogicItem_Knowledge,
    BotLogicItem_Safety,
    BotLogicItem_Segments,
    BotLogicItem_User,
)
from .bot_logic_knowledge_detail import BotLogicKnowledgeDetail
from .bot_logic_knowledge_item import BotLogicKnowledgeItem
from .bot_logic_safety_item import BotLogicSafetyItem
from .bot_logic_segments_item import BotLogicSegmentsItem
from .bot_logic_user_item import BotLogicUserItem
from .bot_message import BotMessage
from .bot_o_auth_button_response import BotOAuthButtonResponse
from .bot_object_response import BotObjectResponse
from .bot_response import (
    BotResponse,
    BotResponse_ActionForm,
    BotResponse_Chart,
    BotResponse_OauthButton,
    BotResponse_Object,
    BotResponse_Text,
)
from .bot_response_metadata import BotResponseMetadata
from .bot_text_response import BotTextResponse
from .browser_info import BrowserInfo
from .browser_type import BrowserType
from .capability import Capability
from .chart_spec_schema import ChartSpecSchema
from .context_info import ContextInfo
from .conversation_analysis import ConversationAnalysis
from .conversation_executed_action_precondition import ConversationExecutedActionPrecondition
from .conversation_information import ConversationInformation
from .conversation_message_base import ConversationMessageBase
from .conversation_message_response import (
    ConversationMessageResponse,
    ConversationMessageResponse_Bot,
    ConversationMessageResponse_User,
)
from .conversation_precondition import (
    ConversationPrecondition,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_App,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_ResponseConfig,
    ConversationPrecondition_Tags,
)
from .conversation_preview import ConversationPreview
from .conversation_response import ConversationResponse
from .conversation_summary import ConversationSummary
from .device_info import DeviceInfo
from .device_type import DeviceType
from .document_information import DocumentInformation
from .entity_id import EntityId
from .entity_id_base import EntityIdBase
from .entity_id_filter import EntityIdFilter
from .entity_id_without_agent import EntityIdWithoutAgent
from .entity_result import EntityResult
from .entity_type import EntityType
from .error_message import ErrorMessage
from .event_base_no_id import EventBaseNoId
from .event_field import EventField
from .event_filter import EventFilter
from .event_request import EventRequest, EventRequest_SystemEvent, EventRequest_UserEvent
from .event_response import EventResponse, EventResponse_SystemEvent, EventResponse_UserEvent
from .event_type import EventType
from .event_user_info import EventUserInfo
from .event_user_info_base import EventUserInfoBase
from .events_search_request import EventsSearchRequest
from .events_search_response import EventsSearchResponse
from .feedback import Feedback
from .feedback_base import FeedbackBase
from .feedback_info import FeedbackInfo
from .feedback_type import FeedbackType
from .geo_info import GeoInfo
from .inbox_item import InboxItem, InboxItem_Custom, InboxItem_DuplicateDocuments, InboxItem_MissingKnowledge
from .inbox_item_base import InboxItemBase
from .inbox_item_custom import InboxItemCustom
from .inbox_item_duplicate_documents import InboxItemDuplicateDocuments
from .inbox_item_fix import InboxItemFix, InboxItemFix_AddDocument, InboxItemFix_DeactivateDocument
from .inbox_item_fix_add_document import InboxItemFixAddDocument
from .inbox_item_fix_base import InboxItemFixBase
from .inbox_item_fix_deactivate_document import InboxItemFixDeactivateDocument
from .inbox_item_missing_knowledge import InboxItemMissingKnowledge
from .inbox_item_severity import InboxItemSeverity
from .inbox_item_status import InboxItemStatus
from .inbox_item_type import InboxItemType
from .ip_info import IpInfo
from .language_info import LanguageInfo
from .llm_inclusion_status import LlmInclusionStatus
from .llm_persona import LlmPersona
from .message_status import MessageStatus
from .metadata_precondition import MetadataPrecondition
from .novel_system_event import NovelSystemEvent
from .novel_system_event_no_id import NovelSystemEventNoId
from .novel_user_event import NovelUserEvent
from .number_range import NumberRange
from .os_info import OsInfo
from .os_type import OsType
from .page import Page
from .page_info import PageInfo
from .precondition import Precondition, Precondition_Conversation, Precondition_Group, Precondition_User
from .precondition_base import PreconditionBase
from .precondition_group import PreconditionGroup
from .precondition_group_operator import PreconditionGroupOperator
from .precondition_operator import PreconditionOperator
from .quality import Quality
from .quality_reason import QualityReason
from .response_config import ResponseConfig
from .response_config_precondition import ResponseConfigPrecondition
from .response_length import ResponseLength
from .scoped_entity import ScopedEntity
from .sentiment import Sentiment
from .session_info import SessionInfo
from .simulation_context import SimulationContext
from .source import Source
from .source_info import SourceInfo
from .source_type import SourceType
from .survey_info import SurveyInfo
from .system_event import SystemEvent
from .system_event_name import SystemEventName
from .tags_precondition import TagsPrecondition
from .user_conversation_message_type import UserConversationMessageType
from .user_data import UserData
from .user_data_with_reference import UserDataWithReference
from .user_event import UserEvent
from .user_event_name import UserEventName
from .user_message import UserMessage
from .user_message_base import UserMessageBase
from .user_message_response_state import UserMessageResponseState
from .visibility_type import VisibilityType

__all__ = [
    "ActionBase",
    "ActionEnumOption",
    "ActionExecutionParamValue",
    "ActionExecutionParamValue_Attachment",
    "ActionExecutionParamValue_Primitive",
    "ActionFormField",
    "ActionOAuthConfiguration",
    "ActionParameter",
    "ActionParameterType",
    "ActionResponse",
    "AppPrecondition",
    "AppUser",
    "AppUserIdentifier",
    "AppUserIdentifyingPropertyType",
    "AppUserRequest",
    "AppUserResponse",
    "AttachmentRequest",
    "AttachmentResponse",
    "AttachmentStatus",
    "BaseAttachment",
    "BaseConversationResponse",
    "BasePaginatedRequest",
    "BaseSearchRequest",
    "BotActionFormResponse",
    "BotChartResponse",
    "BotConversationMessageType",
    "BotLogic",
    "BotLogicActionExecutedDetail",
    "BotLogicActionItem",
    "BotLogicActionReviewedDetail",
    "BotLogicFormSubmissionItem",
    "BotLogicItem",
    "BotLogicItem_Actions",
    "BotLogicItem_Form",
    "BotLogicItem_Knowledge",
    "BotLogicItem_Safety",
    "BotLogicItem_Segments",
    "BotLogicItem_User",
    "BotLogicKnowledgeDetail",
    "BotLogicKnowledgeItem",
    "BotLogicSafetyItem",
    "BotLogicSegmentsItem",
    "BotLogicUserItem",
    "BotMessage",
    "BotOAuthButtonResponse",
    "BotObjectResponse",
    "BotResponse",
    "BotResponseMetadata",
    "BotResponse_ActionForm",
    "BotResponse_Chart",
    "BotResponse_OauthButton",
    "BotResponse_Object",
    "BotResponse_Text",
    "BotTextResponse",
    "BrowserInfo",
    "BrowserType",
    "Capability",
    "ChartSpecSchema",
    "ContextInfo",
    "ConversationAnalysis",
    "ConversationExecutedActionPrecondition",
    "ConversationInformation",
    "ConversationMessageBase",
    "ConversationMessageResponse",
    "ConversationMessageResponse_Bot",
    "ConversationMessageResponse_User",
    "ConversationPrecondition",
    "ConversationPrecondition_ActionExecuted",
    "ConversationPrecondition_App",
    "ConversationPrecondition_Metadata",
    "ConversationPrecondition_ResponseConfig",
    "ConversationPrecondition_Tags",
    "ConversationPreview",
    "ConversationResponse",
    "ConversationSummary",
    "DeviceInfo",
    "DeviceType",
    "DocumentInformation",
    "EntityId",
    "EntityIdBase",
    "EntityIdFilter",
    "EntityIdWithoutAgent",
    "EntityResult",
    "EntityType",
    "ErrorMessage",
    "EventBaseNoId",
    "EventField",
    "EventFilter",
    "EventRequest",
    "EventRequest_SystemEvent",
    "EventRequest_UserEvent",
    "EventResponse",
    "EventResponse_SystemEvent",
    "EventResponse_UserEvent",
    "EventType",
    "EventUserInfo",
    "EventUserInfoBase",
    "EventsSearchRequest",
    "EventsSearchResponse",
    "Feedback",
    "FeedbackBase",
    "FeedbackInfo",
    "FeedbackType",
    "GeoInfo",
    "InboxItem",
    "InboxItemBase",
    "InboxItemCustom",
    "InboxItemDuplicateDocuments",
    "InboxItemFix",
    "InboxItemFixAddDocument",
    "InboxItemFixBase",
    "InboxItemFixDeactivateDocument",
    "InboxItemFix_AddDocument",
    "InboxItemFix_DeactivateDocument",
    "InboxItemMissingKnowledge",
    "InboxItemSeverity",
    "InboxItemStatus",
    "InboxItemType",
    "InboxItem_Custom",
    "InboxItem_DuplicateDocuments",
    "InboxItem_MissingKnowledge",
    "IpInfo",
    "LanguageInfo",
    "LlmInclusionStatus",
    "LlmPersona",
    "MessageStatus",
    "MetadataPrecondition",
    "NovelSystemEvent",
    "NovelSystemEventNoId",
    "NovelUserEvent",
    "NumberRange",
    "OsInfo",
    "OsType",
    "Page",
    "PageInfo",
    "Precondition",
    "PreconditionBase",
    "PreconditionGroup",
    "PreconditionGroupOperator",
    "PreconditionOperator",
    "Precondition_Conversation",
    "Precondition_Group",
    "Precondition_User",
    "Quality",
    "QualityReason",
    "ResponseConfig",
    "ResponseConfigPrecondition",
    "ResponseLength",
    "ScopedEntity",
    "Sentiment",
    "SessionInfo",
    "SimulationContext",
    "Source",
    "SourceInfo",
    "SourceType",
    "SurveyInfo",
    "SystemEvent",
    "SystemEventName",
    "TagsPrecondition",
    "UserConversationMessageType",
    "UserData",
    "UserDataWithReference",
    "UserEvent",
    "UserEventName",
    "UserMessage",
    "UserMessageBase",
    "UserMessageResponseState",
    "VisibilityType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .conversation_analytics_request import ConversationAnalyticsRequest
from .conversation_group_by import ConversationGroupBy
from .conversation_metric import ConversationMetric


class ConversationBarChartRequest(ConversationAnalyticsRequest):
    bar_definition: typing_extensions.Annotated[ConversationGroupBy, FieldMetadata(alias="barDefinition")] = (
        pydantic.Field()
    )
    """
    Determines how data is grouped along the x-axis. Each unique value forms a separate bar.
    The name of the bar is derived from the grouping field's value or range.
    
    **Label Formats by Field:**
    - `Sources`: Document title (e.g., "Product Documentation")
    - `Actions`: "Action Name - App Name" (e.g., "Create Ticket - Freshdesk")
    - `IncompleteActions`: "Action Name - App Name" (e.g., "Search Documents - Zendesk")
    - `Users`: User identifier value (e.g., "user@example.com")
    - `InboxItems`: Inbox item title
    - `HumanAgents`: Human agent display name
    - `HumanAgentsWithInserts`: Human agent display name (for agents who made inserts)
    - Other fields: Field value as stored (e.g., "GOOD", "NEEDS_IMPROVEMENT" for Quality)
    """

    metric: ConversationMetric = pydantic.Field()
    """
    Metric defining the y-axis values for the bar chart.
    """

    vertical_grouping: typing_extensions.Annotated[
        typing.Optional[ConversationGroupBy], FieldMetadata(alias="verticalGrouping")
    ] = pydantic.Field(default=None)
    """
    Optionally defines vertical grouping within each bar, producing multiple series.
    If omitted, a single series is generated.
    
    **Series Name Formats by Field:**
    - `Sources`: Document title (e.g., "Product Documentation")
    - `Actions`: "Action Name - App Name" (e.g., "Create Ticket - Freshdesk")
    - `IncompleteActions`: "Action Name - App Name" (e.g., "Search Documents - Zendesk")
    - `Users`: User identifier value (e.g., "user@example.com")
    - `InboxItems`: Inbox item title
    - `HumanAgents`: Human agent display name
    - `HumanAgentsWithInserts`: Human agent display name (for agents who made inserts)
    - Other fields: Field value as stored (e.g., "GOOD", "NEEDS_IMPROVEMENT" for Quality)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    BaseKnowledgeDocument,
    FinalizeKnowledgeBaseVersionRequest,
    KnowledgeBaseField,
    KnowledgeBaseFilter,
    KnowledgeBaseProperties,
    KnowledgeBaseRefreshFrequency,
    KnowledgeBaseRefreshRequest,
    KnowledgeBaseRequest,
    KnowledgeBaseResponse,
    KnowledgeBaseSearchRequest,
    KnowledgeBaseType,
    KnowledgeBaseVersion,
    KnowledgeBaseVersionFinalizeStatus,
    KnowledgeBaseVersionRequest,
    KnowledgeBaseVersionStatus,
    KnowledgeBaseVersionType,
    KnowledgeBaseVersionsListResponse,
    KnowledgeBasesResponse,
    KnowledgeDeleteRequest,
    KnowledgeDocumentContentType,
    KnowledgeDocumentField,
    KnowledgeDocumentFilter,
    KnowledgeDocumentRequest,
    KnowledgeDocumentResponse,
    KnowledgeDocumentSearchRequest,
    KnowledgeDocumentSearchResponse,
    KnowledgeDocumentStatus,
    KnowledgeDocumentsResponse,
)

__all__ = [
    "BaseKnowledgeDocument",
    "FinalizeKnowledgeBaseVersionRequest",
    "KnowledgeBaseField",
    "KnowledgeBaseFilter",
    "KnowledgeBaseProperties",
    "KnowledgeBaseRefreshFrequency",
    "KnowledgeBaseRefreshRequest",
    "KnowledgeBaseRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseSearchRequest",
    "KnowledgeBaseType",
    "KnowledgeBaseVersion",
    "KnowledgeBaseVersionFinalizeStatus",
    "KnowledgeBaseVersionRequest",
    "KnowledgeBaseVersionStatus",
    "KnowledgeBaseVersionType",
    "KnowledgeBaseVersionsListResponse",
    "KnowledgeBasesResponse",
    "KnowledgeDeleteRequest",
    "KnowledgeDocumentContentType",
    "KnowledgeDocumentField",
    "KnowledgeDocumentFilter",
    "KnowledgeDocumentRequest",
    "KnowledgeDocumentResponse",
    "KnowledgeDocumentSearchRequest",
    "KnowledgeDocumentSearchResponse",
    "KnowledgeDocumentStatus",
    "KnowledgeDocumentsResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import CommitAssetUploadRequest, InitiateAssetUploadRequest, InitiateAssetUploadResponse

__all__ = ["CommitAssetUploadRequest", "InitiateAssetUploadRequest", "InitiateAssetUploadResponse"]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .app_settings import AppSettings
from .search_app_settings_response import SearchAppSettingsResponse

__all__ = ["AppSettings", "SearchAppSettingsResponse"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_without_agent import EntityIdWithoutAgent
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class KnowledgeDeleteRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdWithoutAgent
    from mavenagi.knowledge import KnowledgeDeleteRequest

    KnowledgeDeleteRequest(
        version_id=EntityIdWithoutAgent(
            type="KNOWLEDGE_BASE_VERSION",
            app_id="maven",
            reference_id="versionId",
        ),
    )
    """

    version_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="versionId")] = pydantic.Field()
    """
    ID that uniquely identifies which knowledge base version to delete the document from.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import AddDocumentFixRequest, ApplyFixesRequest, InboxFilter, InboxSearchRequest, InboxSearchResponse

__all__ = ["AddDocumentFixRequest", "ApplyFixesRequest", "InboxFilter", "InboxSearchRequest", "InboxSearchResponse"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id import EntityId


class EventUserInfo(UniversalBaseModel):
    id: typing.Optional[EntityId] = None
    user_display_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="userDisplayName")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class EntityIdBase(UniversalBaseModel):
    """
    All other entityId fields are inferred from the API request.
    """

    reference_id: typing_extensions.Annotated[str, FieldMetadata(alias="referenceId")] = pydantic.Field()
    """
    Externally supplied ID to uniquely identify this object. Is globally unique when combined with all other entityId fields (type, appId, organizationId, agentId).
    
    Must be less than 192 characters and contain only:
    - alphanumeric characters (`a-z`, `A-Z`, `0-9`)
    - hyphens (`-`)
    - underscores (`_`)
    - plus signs (`+`)
    - periods (`.`)
    - at symbol (`@`)
    - pipe symbol (`|`)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.app_user_identifier import AppUserIdentifier
from ..commons.types.app_user_response import AppUserResponse
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.user_data import UserData
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawUsersClient, RawUsersClient
from .types.agent_user import AgentUser
from .types.agent_user_field import AgentUserField
from .types.agent_user_filter import AgentUserFilter
from .types.agent_user_search_response import AgentUserSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def search(
        self,
        *,
        sort: typing.Optional[AgentUserField] = OMIT,
        filter: typing.Optional[AgentUserFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentUserSearchResponse:
        """
        Search across all agent users on an agent.

        Agent users are a merged view of the users created by individual apps.

        Parameters
        ----------
        sort : typing.Optional[AgentUserField]

        filter : typing.Optional[AgentUserFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUserSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def get_agent_user(
        self, agent_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentUser:
        """
        Get an agent user by its supplied ID.

        Agent users are a merged view of the users created by individual apps.

        Parameters
        ----------
        agent_user_id : str
            The ID of the agent user to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUser

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.get_agent_user(
            agent_user_id="aus_1234567890",
        )
        """
        _response = self._raw_client.get_agent_user(agent_user_id, request_options=request_options)
        return _response.data

    def create_or_update(
        self,
        *,
        user_id: EntityIdBase,
        identifiers: typing.Sequence[AppUserIdentifier],
        data: typing.Dict[str, UserData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Update an app user or create it if it doesn't exist.

        Parameters
        ----------
        user_id : EntityIdBase
            ID that uniquely identifies this app user

        identifiers : typing.Sequence[AppUserIdentifier]
            Used to determine whether two users from different apps are the same

        data : typing.Dict[str, UserData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import AppUserIdentifier, EntityIdBase, UserData

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.create_or_update(
            user_id=EntityIdBase(
                reference_id="user-0",
            ),
            identifiers=[
                AppUserIdentifier(
                    value="joe@myapp.com",
                    type="EMAIL",
                )
            ],
            data={
                "name": UserData(
                    value="Joe",
                    visibility="VISIBLE",
                )
            },
        )
        """
        _response = self._raw_client.create_or_update(
            user_id=user_id, identifiers=identifiers, data=data, request_options=request_options
        )
        return _response.data

    def get(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Get an app user by its supplied ID

        Parameters
        ----------
        user_id : str
            The reference ID of the app user to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the app user to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.get(
            user_id="user-0",
        )
        """
        _response = self._raw_client.get(user_id, app_id=app_id, request_options=request_options)
        return _response.data

    def delete(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all identifiers and user data saved by the specified app.
        Does not modify data or identifiers saved by other apps.

        If this user is linked to a user from another app, it will not be unlinked. Unlinking of users is not yet supported.

        <Warning>This is a destructive operation and cannot be undone.</Warning>

        Parameters
        ----------
        user_id : str
            The reference ID of the app user to delete. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the app user to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.delete(
            user_id="user-0",
        )
        """
        _response = self._raw_client.delete(user_id, app_id=app_id, request_options=request_options)
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        sort: typing.Optional[AgentUserField] = OMIT,
        filter: typing.Optional[AgentUserFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentUserSearchResponse:
        """
        Search across all agent users on an agent.

        Agent users are a merged view of the users created by individual apps.

        Parameters
        ----------
        sort : typing.Optional[AgentUserField]

        filter : typing.Optional[AgentUserFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUserSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def get_agent_user(
        self, agent_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentUser:
        """
        Get an agent user by its supplied ID.

        Agent users are a merged view of the users created by individual apps.

        Parameters
        ----------
        agent_user_id : str
            The ID of the agent user to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUser

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.get_agent_user(
                agent_user_id="aus_1234567890",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_agent_user(agent_user_id, request_options=request_options)
        return _response.data

    async def create_or_update(
        self,
        *,
        user_id: EntityIdBase,
        identifiers: typing.Sequence[AppUserIdentifier],
        data: typing.Dict[str, UserData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Update an app user or create it if it doesn't exist.

        Parameters
        ----------
        user_id : EntityIdBase
            ID that uniquely identifies this app user

        identifiers : typing.Sequence[AppUserIdentifier]
            Used to determine whether two users from different apps are the same

        data : typing.Dict[str, UserData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import AppUserIdentifier, EntityIdBase, UserData

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.create_or_update(
                user_id=EntityIdBase(
                    reference_id="user-0",
                ),
                identifiers=[
                    AppUserIdentifier(
                        value="joe@myapp.com",
                        type="EMAIL",
                    )
                ],
                data={
                    "name": UserData(
                        value="Joe",
                        visibility="VISIBLE",
                    )
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            user_id=user_id, identifiers=identifiers, data=data, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Get an app user by its supplied ID

        Parameters
        ----------
        user_id : str
            The reference ID of the app user to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the app user to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.get(
                user_id="user-0",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(user_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def delete(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all identifiers and user data saved by the specified app.
        Does not modify data or identifiers saved by other apps.

        If this user is linked to a user from another app, it will not be unlinked. Unlinking of users is not yet supported.

        <Warning>This is a destructive operation and cannot be undone.</Warning>

        Parameters
        ----------
        user_id : str
            The reference ID of the app user to delete. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the app user to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.delete(
                user_id="user-0",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(user_id, app_id=app_id, request_options=request_options)
        return _response.data
